@@ -10,10 +10,10 @@
 ///
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Last changed  : $Date: 2006/03/19 10:05:49 $
-// File revision : $Revision: 1.13 $
+// Last changed  : $Date: 2009-10-31 16:37:24 +0200 (Sat, 31 Oct 2009) $
+// File revision : $Revision: 4 $
 //
-// $Id: RateTransposer.cpp,v 1.13 2006/03/19 10:05:49 Olli Exp $
+// $Id: RateTransposer.cpp 74 2009-10-31 14:37:24Z oparviai $
 //
 ////////////////////////////////////////////////////////////////////////////////
 //
@@ -42,10 +42,11 @@
 #include <assert.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <limits.h>
+#include <stdexcept>
 #include "RateTransposer.h"
 #include "AAFilter.h"
 
+using namespace std;
 using namespace soundtouch;
 
 
@@ -55,7 +56,7 @@ class RateTransposerInteger : public RateTransposer
 {
 protected:
     int iSlopeCount;
-    uint uRate;
+    int iRate;
     SAMPLETYPE sPrevSampleL, sPrevSampleR;
 
     virtual void resetRegisters();
@@ -84,7 +85,6 @@ class RateTransposerFloat : public RateTransposer
 {
 protected:
     float fSlopeCount;
-    float fRateStep;
     SAMPLETYPE sPrevSampleL, sPrevSampleR;
 
     virtual void resetRegisters();
@@ -103,18 +103,12 @@ public:
 
 
 
-#ifndef min
-#define min(a,b) ((a > b) ? b : a)
-#define max(a,b) ((a < b) ? b : a)
-#endif
-
 
 // Operator 'new' is overloaded so that it automatically creates a suitable instance 
 // depending on if we've a MMX/SSE/etc-capable CPU available or not.
 void * RateTransposer::operator new(size_t s)
 {
-    // Notice! don't use "new TDStretch" directly, use "newInstance" to create a new instance instead!
-    assert(FALSE);  
+    throw runtime_error("Error in RateTransoser::new: don't use \"new TDStretch\" directly, use \"newInstance\" to create a new instance instead!");
     return NULL;
 }
 
@@ -132,8 +126,9 @@ RateTransposer *RateTransposer::newInstance()
 // Constructor
 RateTransposer::RateTransposer() : FIFOProcessor(&outputBuffer)
 {
-    uChannels = 2;
+    numChannels = 2;
     bUseAAFilter = TRUE;
+    fRate = 0;
 
     // Instantiates the anti-alias filter with default tap length
     // of 32
@@ -150,7 +145,7 @@ RateTransposer::~RateTransposer()
 
 
 /// Enables/disables the anti-alias filter. Zero to disable, nonzero to enable
-void RateTransposer::enableAAFilter(const BOOL newMode)
+void RateTransposer::enableAAFilter(BOOL newMode)
 {
     bUseAAFilter = newMode;
 }
@@ -163,18 +158,18 @@ BOOL RateTransposer::isAAFilterEnabled() const
 }
 
 
-AAFilter *RateTransposer::getAAFilter() const
+AAFilter *RateTransposer::getAAFilter()
 {
     return pAAFilter;
 }
 
 
 
-// Sets new target uRate. Normal uRate = 1.0, smaller values represent slower 
-// uRate, larger faster uRates.
+// Sets new target iRate. Normal iRate = 1.0, smaller values represent slower 
+// iRate, larger faster iRates.
 void RateTransposer::setRate(float newRate)
 {
-    float fCutoff;
+    double fCutoff;
 
     fRate = newRate;
 
@@ -197,45 +192,47 @@ void RateTransposer::setRate(float newRate)
 //
 // It's allowed for 'output' and 'input' parameters to point to the same
 // memory position.
+/*
 void RateTransposer::flushStoreBuffer()
 {
     if (storeBuffer.isEmpty()) return;
 
     outputBuffer.moveSamples(storeBuffer);
 }
+*/
 
 
-// Adds 'numSamples' pcs of samples from the 'samples' memory position into
+// Adds 'nSamples' pcs of samples from the 'samples' memory position into
 // the input of the object.
-void RateTransposer::putSamples(const SAMPLETYPE *samples, uint numSamples)
+void RateTransposer::putSamples(const SAMPLETYPE *samples, uint nSamples)
 {
-    processSamples(samples, numSamples);
+    processSamples(samples, nSamples);
 }
 
 
 
 // Transposes up the sample rate, causing the observed playback 'rate' of the
 // sound to decrease
-void RateTransposer::upsample(const SAMPLETYPE *src, uint numSamples)
+void RateTransposer::upsample(const SAMPLETYPE *src, uint nSamples)
 {
-    int count, sizeTemp, num;
+    uint count, sizeTemp, num;
 
     // If the parameter 'uRate' value is smaller than 'SCALE', first transpose
     // the samples and then apply the anti-alias filter to remove aliasing.
 
     // First check that there's enough room in 'storeBuffer' 
     // (+16 is to reserve some slack in the destination buffer)
-    sizeTemp = (int)((float)numSamples / fRate + 16.0f);
+    sizeTemp = (uint)((float)nSamples / fRate + 16.0f);
 
     // Transpose the samples, store the result into the end of "storeBuffer"
-    count = transpose(storeBuffer.ptrEnd(sizeTemp), src, numSamples);
+    count = transpose(storeBuffer.ptrEnd(sizeTemp), src, nSamples);
     storeBuffer.putSamples(count);
 
     // Apply the anti-alias filter to samples in "store output", output the
     // result to "dest"
     num = storeBuffer.numSamples();
     count = pAAFilter->evaluate(outputBuffer.ptrEnd(num), 
-        storeBuffer.ptrBegin(), num, uChannels);
+        storeBuffer.ptrBegin(), num, (uint)numChannels);
     outputBuffer.putSamples(count);
 
     // Remove the processed samples from "storeBuffer"
@@ -245,16 +242,16 @@ void RateTransposer::upsample(const SAMPLETYPE *src, uint numSamples)
 
 // Transposes down the sample rate, causing the observed playback 'rate' of the
 // sound to increase
-void RateTransposer::downsample(const SAMPLETYPE *src, uint numSamples)
+void RateTransposer::downsample(const SAMPLETYPE *src, uint nSamples)
 {
-    int count, sizeTemp;
+    uint count, sizeTemp;
 
     // If the parameter 'uRate' value is larger than 'SCALE', first apply the
     // anti-alias filter to remove high frequencies (prevent them from folding
-    // over the lover frequencies), then transpose. */
+    // over the lover frequencies), then transpose.
 
-    // Add the new samples to the end of the storeBuffer */
-    storeBuffer.putSamples(src, numSamples);
+    // Add the new samples to the end of the storeBuffer
+    storeBuffer.putSamples(src, nSamples);
 
     // Anti-alias filter the samples to prevent folding and output the filtered 
     // data to tempBuffer. Note : because of the FIR filter length, the
@@ -263,13 +260,15 @@ void RateTransposer::downsample(const SAMPLETYPE *src, uint numSamples)
     sizeTemp = storeBuffer.numSamples();
 
     count = pAAFilter->evaluate(tempBuffer.ptrEnd(sizeTemp), 
-        storeBuffer.ptrBegin(), sizeTemp, uChannels);
+        storeBuffer.ptrBegin(), sizeTemp, (uint)numChannels);
+
+	if (count == 0) return;
 
     // Remove the filtered samples from 'storeBuffer'
     storeBuffer.receiveSamples(count);
 
     // Transpose the samples (+16 is to reserve some slack in the destination buffer)
-    sizeTemp = (int)((float)numSamples / fRate + 16.0f);
+    sizeTemp = (uint)((float)nSamples / fRate + 16.0f);
     count = transpose(outputBuffer.ptrEnd(sizeTemp), tempBuffer.ptrBegin(), count);
     outputBuffer.putSamples(count);
 }
@@ -279,20 +278,20 @@ void RateTransposer::downsample(const SAMPLETYPE *src, uint numSamples)
 // Returns amount of samples returned in the "dest" buffer.
 // The maximum amount of samples that can be returned at a time is set by
 // the 'set_returnBuffer_size' function.
-void RateTransposer::processSamples(const SAMPLETYPE *src, uint numSamples)
+void RateTransposer::processSamples(const SAMPLETYPE *src, uint nSamples)
 {
     uint count;
     uint sizeReq;
 
-    if (numSamples == 0) return;
+    if (nSamples == 0) return;
     assert(pAAFilter);
 
     // If anti-alias filter is turned off, simply transpose without applying
     // the filter
     if (bUseAAFilter == FALSE) 
     {
-        sizeReq = (int)((float)numSamples / fRate + 1.0f);
-        count = transpose(outputBuffer.ptrEnd(sizeReq), src, numSamples);
+        sizeReq = (uint)((float)nSamples / fRate + 1.0f);
+        count = transpose(outputBuffer.ptrEnd(sizeReq), src, nSamples);
         outputBuffer.putSamples(count);
         return;
     }
@@ -300,41 +299,42 @@ void RateTransposer::processSamples(const SAMPLETYPE *src, uint numSamples)
     // Transpose with anti-alias filter
     if (fRate < 1.0f) 
     {
-        upsample(src, numSamples);
+        upsample(src, nSamples);
     } 
     else  
     {
-        downsample(src, numSamples);
+        downsample(src, nSamples);
     }
 }
 
 
 // Transposes the sample rate of the given samples using linear interpolation. 
 // Returns the number of samples returned in the "dest" buffer
-inline uint RateTransposer::transpose(SAMPLETYPE *dest, const SAMPLETYPE *src, uint numSamples)
+inline uint RateTransposer::transpose(SAMPLETYPE *dest, const SAMPLETYPE *src, uint nSamples)
 {
-    if (uChannels == 2) 
+    if (numChannels == 2) 
     {
-        return transposeStereo(dest, src, numSamples);
+        return transposeStereo(dest, src, nSamples);
     } 
     else 
     {
-        return transposeMono(dest, src, numSamples);
+        return transposeMono(dest, src, nSamples);
     }
 }
 
 
 // Sets the number of channels, 1 = mono, 2 = stereo
-void RateTransposer::setChannels(const uint numchannels)
+void RateTransposer::setChannels(int nChannels)
 {
-    if (uChannels == numchannels) return;
+    assert(nChannels > 0);
+    if (numChannels == nChannels) return;
 
-    assert(numchannels == 1 || numchannels == 2);
-    uChannels = numchannels;
+    assert(nChannels == 1 || nChannels == 2);
+    numChannels = nChannels;
 
-    storeBuffer.setChannels(uChannels);
-    tempBuffer.setChannels(uChannels);
-    outputBuffer.setChannels(uChannels);
+    storeBuffer.setChannels(numChannels);
+    tempBuffer.setChannels(numChannels);
+    outputBuffer.setChannels(numChannels);
 
     // Inits the linear interpolation registers
     resetRegisters();
@@ -350,7 +350,7 @@ void RateTransposer::clear()
 
 
 // Returns nonzero if there aren't any samples available for outputting.
-uint RateTransposer::isEmpty()
+int RateTransposer::isEmpty() const
 {
     int res;
 
@@ -371,11 +371,10 @@ uint RateTransposer::isEmpty()
 // Constructor
 RateTransposerInteger::RateTransposerInteger() : RateTransposer()
 {
-    // call these here as these are virtual functions; calling these
-    // from the base class constructor wouldn't execute the overloaded
-    // versions (<master yoda>peculiar C++ can be</my>).
-    resetRegisters();
-    setRate(1.0f);
+    // Notice: use local function calling syntax for sake of clarity, 
+    // to indicate the fact that C++ constructor can't call virtual functions.
+    RateTransposerInteger::resetRegisters();
+    RateTransposerInteger::setRate(1.0f);
 }
 
 
@@ -396,12 +395,14 @@ void RateTransposerInteger::resetRegisters()
 // Transposes the sample rate of the given samples using linear interpolation. 
 // 'Mono' version of the routine. Returns the number of samples returned in 
 // the "dest" buffer
-uint RateTransposerInteger::transposeMono(SAMPLETYPE *dest, const SAMPLETYPE *src, uint numSamples)
+uint RateTransposerInteger::transposeMono(SAMPLETYPE *dest, const SAMPLETYPE *src, uint nSamples)
 {
     unsigned int i, used;
     LONG_SAMPLETYPE temp, vol1;
 
-    used = 0;    
+    if (nSamples == 0) return 0;  // no samples, no work
+
+	used = 0;    
     i = 0;
 
     // Process the last sample saved from the previous call first...
@@ -411,7 +412,7 @@ uint RateTransposerInteger::transposeMono(SAMPLETYPE *dest, const SAMPLETYPE *sr
         temp = vol1 * sPrevSampleL + iSlopeCount * src[0];
         dest[i] = (SAMPLETYPE)(temp / SCALE);
         i++;
-        iSlopeCount += uRate;
+        iSlopeCount += iRate;
     }
     // now always (iSlopeCount > SCALE)
     iSlopeCount -= SCALE;
@@ -422,18 +423,18 @@ uint RateTransposerInteger::transposeMono(SAMPLETYPE *dest, const SAMPLETYPE *sr
         {
             iSlopeCount -= SCALE;
             used ++;
-            if (used >= numSamples - 1) goto end;
+            if (used >= nSamples - 1) goto end;
         }
         vol1 = (LONG_SAMPLETYPE)(SCALE - iSlopeCount);
         temp = src[used] * vol1 + iSlopeCount * src[used + 1];
         dest[i] = (SAMPLETYPE)(temp / SCALE);
 
         i++;
-        iSlopeCount += uRate;
+        iSlopeCount += iRate;
     }
 end:
     // Store the last sample for the next round
-    sPrevSampleL = src[numSamples - 1];
+    sPrevSampleL = src[nSamples - 1];
 
     return i;
 }
@@ -442,12 +443,12 @@ end:
 // Transposes the sample rate of the given samples using linear interpolation. 
 // 'Stereo' version of the routine. Returns the number of samples returned in 
 // the "dest" buffer
-uint RateTransposerInteger::transposeStereo(SAMPLETYPE *dest, const SAMPLETYPE *src, uint numSamples)
+uint RateTransposerInteger::transposeStereo(SAMPLETYPE *dest, const SAMPLETYPE *src, uint nSamples)
 {
     unsigned int srcPos, i, used;
     LONG_SAMPLETYPE temp, vol1;
 
-    if (numSamples == 0) return 0;  // no samples, no work
+    if (nSamples == 0) return 0;  // no samples, no work
 
     used = 0;    
     i = 0;
@@ -461,7 +462,7 @@ uint RateTransposerInteger::transposeStereo(SAMPLETYPE *dest, const SAMPLETYPE *
         temp = vol1 * sPrevSampleR + iSlopeCount * src[1];
         dest[2 * i + 1] = (SAMPLETYPE)(temp / SCALE);
         i++;
-        iSlopeCount += uRate;
+        iSlopeCount += iRate;
     }
     // now always (iSlopeCount > SCALE)
     iSlopeCount -= SCALE;
@@ -472,7 +473,7 @@ uint RateTransposerInteger::transposeStereo(SAMPLETYPE *dest, const SAMPLETYPE *
         {
             iSlopeCount -= SCALE;
             used ++;
-            if (used >= numSamples - 1) goto end;
+            if (used >= nSamples - 1) goto end;
         }
         srcPos = 2 * used;
         vol1 = (LONG_SAMPLETYPE)(SCALE - iSlopeCount);
@@ -482,22 +483,22 @@ uint RateTransposerInteger::transposeStereo(SAMPLETYPE *dest, const SAMPLETYPE *
         dest[2 * i + 1] = (SAMPLETYPE)(temp / SCALE);
 
         i++;
-        iSlopeCount += uRate;
+        iSlopeCount += iRate;
     }
 end:
     // Store the last sample for the next round
-    sPrevSampleL = src[2 * numSamples - 2];
-    sPrevSampleR = src[2 * numSamples - 1];
+    sPrevSampleL = src[2 * nSamples - 2];
+    sPrevSampleR = src[2 * nSamples - 1];
 
     return i;
 }
 
 
-// Sets new target uRate. Normal uRate = 1.0, smaller values represent slower 
-// uRate, larger faster uRates.
+// Sets new target iRate. Normal iRate = 1.0, smaller values represent slower 
+// iRate, larger faster iRates.
 void RateTransposerInteger::setRate(float newRate)
 {
-    uRate = (int)(newRate * SCALE + 0.5f);
+    iRate = (int)(newRate * SCALE + 0.5f);
     RateTransposer::setRate(newRate);
 }
 
@@ -511,11 +512,10 @@ void RateTransposerInteger::setRate(float newRate)
 // Constructor
 RateTransposerFloat::RateTransposerFloat() : RateTransposer()
 {
-    // call these here as these are virtual functions; calling these
-    // from the base class constructor wouldn't execute the overloaded
-    // versions (<master yoda>peculiar C++ can be</my>).
-    resetRegisters();
-    setRate(1.0f);
+    // Notice: use local function calling syntax for sake of clarity, 
+    // to indicate the fact that C++ constructor can't call virtual functions.
+    RateTransposerFloat::resetRegisters();
+    RateTransposerFloat::setRate(1.0f);
 }
 
 
@@ -536,7 +536,7 @@ void RateTransposerFloat::resetRegisters()
 // Transposes the sample rate of the given samples using linear interpolation. 
 // 'Mono' version of the routine. Returns the number of samples returned in 
 // the "dest" buffer
-uint RateTransposerFloat::transposeMono(SAMPLETYPE *dest, const SAMPLETYPE *src, uint numSamples)
+uint RateTransposerFloat::transposeMono(SAMPLETYPE *dest, const SAMPLETYPE *src, uint nSamples)
 {
     unsigned int i, used;
 
@@ -552,23 +552,24 @@ uint RateTransposerFloat::transposeMono(SAMPLETYPE *dest, const SAMPLETYPE *src,
     }
     fSlopeCount -= 1.0f;
 
-    if (numSamples == 1) goto end;
-
-    while (1)
+    if (nSamples > 1)
     {
-        while (fSlopeCount > 1.0f) 
+        while (1)
         {
-            fSlopeCount -= 1.0f;
-            used ++;
-            if (used >= numSamples - 1) goto end;
+            while (fSlopeCount > 1.0f) 
+            {
+                fSlopeCount -= 1.0f;
+                used ++;
+                if (used >= nSamples - 1) goto end;
+            }
+            dest[i] = (SAMPLETYPE)((1.0f - fSlopeCount) * src[used] + fSlopeCount * src[used + 1]);
+            i++;
+            fSlopeCount += fRate;
         }
-        dest[i] = (SAMPLETYPE)((1.0f - fSlopeCount) * src[used] + fSlopeCount * src[used + 1]);
-        i++;
-        fSlopeCount += fRate;
     }
 end:
     // Store the last sample for the next round
-    sPrevSampleL = src[numSamples - 1];
+    sPrevSampleL = src[nSamples - 1];
 
     return i;
 }
@@ -577,11 +578,11 @@ end:
 // Transposes the sample rate of the given samples using linear interpolation. 
 // 'Mono' version of the routine. Returns the number of samples returned in 
 // the "dest" buffer
-uint RateTransposerFloat::transposeStereo(SAMPLETYPE *dest, const SAMPLETYPE *src, uint numSamples)
+uint RateTransposerFloat::transposeStereo(SAMPLETYPE *dest, const SAMPLETYPE *src, uint nSamples)
 {
     unsigned int srcPos, i, used;
 
-    if (numSamples == 0) return 0;  // no samples, no work
+    if (nSamples == 0) return 0;  // no samples, no work
 
     used = 0;    
     i = 0;
@@ -597,30 +598,31 @@ uint RateTransposerFloat::transposeStereo(SAMPLETYPE *dest, const SAMPLETYPE *sr
     // now always (iSlopeCount > 1.0f)
     fSlopeCount -= 1.0f;
 
-    if (numSamples == 1) goto end;
-
-    while (1)
+    if (nSamples > 1)
     {
-        while (fSlopeCount > 1.0f) 
+        while (1)
         {
-            fSlopeCount -= 1.0f;
-            used ++;
-            if (used >= numSamples - 1) goto end;
+            while (fSlopeCount > 1.0f) 
+            {
+                fSlopeCount -= 1.0f;
+                used ++;
+                if (used >= nSamples - 1) goto end;
+            }
+            srcPos = 2 * used;
+
+            dest[2 * i] = (SAMPLETYPE)((1.0f - fSlopeCount) * src[srcPos] 
+                + fSlopeCount * src[srcPos + 2]);
+            dest[2 * i + 1] = (SAMPLETYPE)((1.0f - fSlopeCount) * src[srcPos + 1] 
+                + fSlopeCount * src[srcPos + 3]);
+
+            i++;
+            fSlopeCount += fRate;
         }
-        srcPos = 2 * used;
-
-        dest[2 * i] = (SAMPLETYPE)((1.0f - fSlopeCount) * src[srcPos] 
-            + fSlopeCount * src[srcPos + 2]);
-        dest[2 * i + 1] = (SAMPLETYPE)((1.0f - fSlopeCount) * src[srcPos + 1] 
-            + fSlopeCount * src[srcPos + 3]);
-
-        i++;
-        fSlopeCount += fRate;
     }
 end:
     // Store the last sample for the next round
-    sPrevSampleL = src[2 * numSamples - 2];
-    sPrevSampleR = src[2 * numSamples - 1];
+    sPrevSampleL = src[2 * nSamples - 2];
+    sPrevSampleR = src[2 * nSamples - 1];
 
     return i;
 }
